# -*- coding: utf-8 -*-
"""
undone add vision
This should take 8 inputs from the transfer function's poisson sources.  Only
one is on at a time, and sometimes none are.  These should start caSized pops,
that in turn start the states for parsing.
The final parse states (move forward, move pyramid, turn right, turn left, stop)
are equivalent to plan goals, though only forward, right, left and stop are
done and are performed in the transfer function.
That's the plan.
"""
# pragma: no cover

__author__ = 'Template'

from hbp_nrp_cle.brainsim import simulator as sim
import numpy as np
import logging

logger = logging.getLogger(__name__)

def initSim():
    #don't call this from NRP.
    sim.setup(timestep=0.1, min_delay=0.1, max_delay=20.0, threads=1, rng_seeds=[1234])


def createMyNeurons():
    CELL_PARAMS = {'v_thresh':-55.0, 'v_reset' : -70.0, 
                                'tau_refrac': 2.0 , 'tau_syn_E': 5.0,  
                                'v_rest' : -65.0,'i_offset':0.0}

    #start, turn, move,left,right,forward,pyramid, stop
    numInputs = 10  #round up from 9 to 10 for 5 CAs
    
    numInputNonCAs = 9
    numCAs = 10
    #start, move, move forward, move pyramid, turn,  0-4
    #turn left, turn right, stop, stop period, 5-9
    #sink 10
    numParseNeurons = numInputs + (numInputNonCAs * CA_SIZE) + (numCAs *CA_SIZE)
    numVisionNeurons = 3
    numNeurons = numParseNeurons + numVisionNeurons
    cells=sim.Population(numNeurons,sim.IF_cond_exp,CELL_PARAMS)

    return cells


def createTwoInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [50.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')

    return [spikeGen0,spikeGen1]


def nealProjection(preNeurons,postNeurons, connectorList,inhExc):
    connList = sim.FromListConnector(connectorList)

    SP = {"weight": 0.005}
    synapse_type = sim.StaticSynapse(**SP)

    sim.Projection(preNeurons, postNeurons, connector= connList,
                   synapse_type=synapse_type,
                   receptor_type=inhExc)   

DELAY = 1.0
CA_SIZE = 5  
INPUT_WEIGHT = 0.15
INTRA_CA_WEIGHT = 0.02  #.03 #0.01 --
HALF_ON_WEIGHT = 0.0008 #0.0017  0.0005--.001
NHALF_ON_WEIGHT = 0.004
FULL_ON_WEIGHT = 0.1 #.02
CA_STOPS_CA_WEIGHT = 0.3# -.3

def turnOnStateFromSpikeSource(spikeSource, toNeurons, toCA):
        connector = []
        for toOffset in range (0,CA_SIZE):
            toNeuron = toOffset + (toCA*CA_SIZE)
            connector = connector + [(0,toNeuron,INPUT_WEIGHT,
                                      DELAY)]
        nealProjection(spikeSource, toNeurons, connector,'excitatory')

def makeCA(neurons, CA):
        #print 'makeCA
        connector = []
        for fromOffset in range (0,CA_SIZE):
            fromNeuron = fromOffset + (CA*CA_SIZE)
            for toOffset in range (0,CA_SIZE):
                toNeuron = toOffset + (CA*CA_SIZE)
                if (toNeuron != fromNeuron):
                    connector = connector + [(fromNeuron,toNeuron,
                                              INTRA_CA_WEIGHT,
                                              DELAY)]

        nealProjection(neurons,neurons,connector,'excitatory')

def turnOnStateFromState(fromNeurons,fromCA,toNeurons,toCA):
    connector = []
    #uses FULL_ON_WEIGHT
    for fromOffset in range (0,CA_SIZE):
        fromNeuron = fromOffset + (fromCA*CA_SIZE)
        for toOffset in range (0,CA_SIZE):
            toNeuron = toOffset + (toCA*CA_SIZE)
            connector=connector+[(fromNeuron,toNeuron,FULL_ON_WEIGHT,DELAY)]

    nealProjection(fromNeurons,toNeurons,connector,'excitatory')



#--Functions when two inputs are needed to turn on a third
#-- Two states are needed to turn on a third.  
#-- This connects one of the inputs to the the third.
#-- Uses HALF_ON_WEIGHT
def stateHalfTurnsOnState(fromNeurons,fromCA,toNeurons,toCA):
    connector = []
    #uses HALF_ON_WEIGHT
    for fromOffset in range (0,CA_SIZE):
        fromNeuron = fromOffset + (fromCA*CA_SIZE)
        for toOffset in range (0,CA_SIZE):
            toNeuron = toOffset + (toCA*CA_SIZE)
            connector=connector+[(fromNeuron,toNeuron,HALF_ON_WEIGHT, DELAY)]

    nealProjection(fromNeurons,toNeurons,connector,'excitatory')

def neuronsHalfTurnsOnState(fromNeurons,fromCA,toNeurons,toCA):
    connector = []
    #uses HALF_ON_WEIGHT
    for fromOffset in range (0,CA_SIZE):
        fromNeuron = fromOffset + (fromCA*CA_SIZE)
        for toOffset in range (0,CA_SIZE):
            toNeuron = toOffset + (toCA*CA_SIZE)
            connector=connector+[(fromNeuron,toNeuron,NHALF_ON_WEIGHT, DELAY)]

    nealProjection(fromNeurons,toNeurons,connector,'excitatory')

#---- One State or other set of neurons turns off another
#-- Uses CA_STOPS_CA_WEIGHT
def stateTurnsOffState(fromNeurons, fromCA, toNeurons, toCA):
    connector = []
    for fromOffset in range (0,CA_SIZE):
        fromNeuron = fromOffset + (fromCA*CA_SIZE)
        for toOffset in range (0,CA_SIZE):
            toNeuron = toOffset + (toCA*CA_SIZE)
            connector = connector + [(fromNeuron,toNeuron,CA_STOPS_CA_WEIGHT,
                                              DELAY)]

    nealProjection(fromNeurons,toNeurons,connector,'inhibitory')

#unused in NRP
def connectPoissonToNeurons(spikeSource,sourceNum, neurons, toCA):
    connector = []
    for toOffset in range (0,2):
        toNeuron = toOffset + (toCA * CA_SIZE)
        connector = connector + [(sourceNum,toNeuron, 2.2, DELAY)]

    nealProjection(spikeSource,neurons,connector,'excitatory')

def turnOnStateFromOneNeuron(fromNeurons,fromNeuron, toNeurons, toCA):
    connector = []
    for toOffset in range (0,CA_SIZE):
        toNeuron = toOffset + (toCA*CA_SIZE)
        connector = connector + [(fromNeuron,toNeuron,INPUT_WEIGHT,DELAY)]

    nealProjection(fromNeurons, toNeurons, connector,'excitatory')



startInput = 0
turnInput = 1
moveInput = 2
leftInput = 3
rightInput = 4
forwardInput = 5
pyramidInput = 6
stopInput = 7
periodInput = 8
startInputNeurons = 2 #The first two are the input neurons
turnInputNeurons = 3
moveInputNeurons = 4
leftInputNeurons = 5
rightInputNeurons = 6
forwardInputNeurons = 7
pyramidInputNeurons = 8
stopInputNeurons = 9
periodInputNeurons = 10
startCA = 11
turnCA = 12
turnLeftCA = 13
turnRightCA = 14
moveCA = 15
moveForwardCA = 16
movePyramidCA = 17
stopCA = 18
stopPeriodCA = 19

#-----Main
circuit = createMyNeurons()

#fire neurons from one input  #not a state
turnOnStateFromOneNeuron(circuit,startInput,circuit,startInputNeurons)
turnOnStateFromOneNeuron(circuit,turnInput,circuit,turnInputNeurons)
turnOnStateFromOneNeuron(circuit,leftInput,circuit,leftInputNeurons)
turnOnStateFromOneNeuron(circuit,rightInput,circuit,rightInputNeurons)
turnOnStateFromOneNeuron(circuit,moveInput,circuit,moveInputNeurons)
turnOnStateFromOneNeuron(circuit,forwardInput,circuit,forwardInputNeurons)
turnOnStateFromOneNeuron(circuit,pyramidInput,circuit,pyramidInputNeurons)
turnOnStateFromOneNeuron(circuit,stopInput,circuit,stopInputNeurons)
turnOnStateFromOneNeuron(circuit,periodInput,circuit,periodInputNeurons)

#start the start state from just the one input
turnOnStateFromState(circuit,startInputNeurons,circuit,startCA)

#make all 8 parse CAs
makeCA(circuit,startCA)
makeCA(circuit,turnCA)
makeCA(circuit,turnLeftCA)
makeCA(circuit,turnRightCA)
makeCA(circuit,moveCA)
makeCA(circuit,moveForwardCA)
makeCA(circuit,movePyramidCA)
makeCA(circuit,stopCA)
makeCA(circuit,stopPeriodCA)

#start -> turn
stateHalfTurnsOnState(circuit,startCA,circuit,turnCA)
neuronsHalfTurnsOnState(circuit,turnInputNeurons,circuit,turnCA)#tIN not a state
stateTurnsOffState(circuit,turnCA,circuit,startCA)

#turn -> turn left
stateHalfTurnsOnState(circuit,turnCA,circuit,turnLeftCA)
neuronsHalfTurnsOnState(circuit,leftInputNeurons,circuit,turnLeftCA)
stateTurnsOffState(circuit,turnLeftCA,circuit,turnCA)

#turn -> turn right
stateHalfTurnsOnState(circuit,turnCA,circuit,turnRightCA)
neuronsHalfTurnsOnState(circuit,rightInputNeurons,circuit,turnRightCA)
stateTurnsOffState(circuit,turnRightCA,circuit,turnCA)

#start -> move
stateHalfTurnsOnState(circuit,startCA,circuit,moveCA)
neuronsHalfTurnsOnState(circuit,moveInputNeurons,circuit,moveCA)
stateTurnsOffState(circuit,moveCA,circuit,startCA)

#move -> move forward
stateHalfTurnsOnState(circuit,moveCA,circuit,moveForwardCA)
neuronsHalfTurnsOnState(circuit,forwardInputNeurons,circuit,moveForwardCA)
stateTurnsOffState(circuit,moveForwardCA,circuit,moveCA)

#move -> move pyramid
stateHalfTurnsOnState(circuit,moveCA,circuit,movePyramidCA)
neuronsHalfTurnsOnState(circuit,pyramidInputNeurons,circuit,movePyramidCA)
stateTurnsOffState(circuit,movePyramidCA,circuit,moveCA)

#start -> stop
stateHalfTurnsOnState(circuit,startCA,circuit,stopCA)
neuronsHalfTurnsOnState(circuit,stopInputNeurons,circuit,stopCA)
stateTurnsOffState(circuit,stopCA,circuit,startCA)

#stop  -> stop period
stateHalfTurnsOnState(circuit,stopCA,circuit,stopPeriodCA)
neuronsHalfTurnsOnState(circuit,periodInputNeurons,circuit,stopPeriodCA)
stateTurnsOffState(circuit,stopPeriodCA,circuit,stopCA)

#start turns off the final states.
stateTurnsOffState(circuit,startCA,circuit,turnLeftCA)
stateTurnsOffState(circuit,startCA,circuit,turnRightCA)
stateTurnsOffState(circuit,startCA,circuit,moveForwardCA)
stateTurnsOffState(circuit,startCA,circuit,movePyramidCA)
stateTurnsOffState(circuit,startCA,circuit,stopPeriodCA)
