# -*- coding: utf-8 -*-
"""
I've taken the code from the nov16/FSAStandAlone.py and the idleBrain.py
I modified crhBrain to get this. Just have one neuron that gets
turned on by the clock in the environment (in the transfer functions).
"""
# pragma: no cover

__author__ = 'Template'

from hbp_nrp_cle.brainsim import simulator as sim
import numpy as np
import logging

logger = logging.getLogger(__name__)

def initSim():
    #don't call this from NRP.
    sim.setup(timestep=0.1, min_delay=0.1, max_delay=20.0, threads=1, rng_seeds=[1234])

def createNeurons():
    # Following parameters were taken from the husky braitenberg brain experiment (braitenberg.py)

    SENSORPARAMS = {'cm': 0.025,
                    'v_rest': -60.5,
                    'tau_m': 10.,
                    'e_rev_E': 0.0,
                    'e_rev_I': -75.0,
                    'v_reset': -60.5,
                    'v_thresh': -60.0,
                    'tau_refrac': 10.0,
                    'tau_syn_E': 2.5,
                    'tau_syn_I': 2.5}

    cell_class = sim.IF_cond_alpha(**SENSORPARAMS)

    # Define the network structure: 2 neurons (1 sensor and 1 actors)
    population = sim.Population(size=2,
                                cellclass=cell_class)

    return population

def createMyNeurons():
    CELL_PARAMS = {'v_thresh':-55.0, 'v_reset' : -70.0, 
                                'tau_refrac': 2.0 , 'tau_syn_E': 5.0,  

                                'v_rest' : -65.0,'i_offset':0.0}
    cells=sim.Population(20,sim.IF_cond_exp,CELL_PARAMS)

    return cells


def createTwoInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [50.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')

    return [spikeGen0,spikeGen1]


def nealProjection(preNeurons,postNeurons, connectorList,inhExc):
    connList = sim.FromListConnector(connectorList)
    #undone this synapse thing might break the fsa 
    SYNAPSE_PARAMS = {"weight": 0.5, #both but less
                      "delay": 20.0,
                      'U': 1.0,
                      'tau_rec': 1.0,
                      'tau_facil': 1.0}
    synapse_type = sim.TsodyksMarkramSynapse(**SYNAPSE_PARAMS)

    sim.Projection(preNeurons, postNeurons, connector= connList,
                   synapse_type=synapse_type,
                   receptor_type=inhExc)   

DELAY = 1.0
CA_SIZE = 5  
INPUT_WEIGHT = 0.1
INTRA_CA_WEIGHT = 0.03 
HALF_ON_WEIGHT = 0.0017 
CA_STOPS_CA_WEIGHT = -0.3 

def turnOnStateFromSpikeSource(spikeSource, toNeurons, toCA):
        connector = []
        for toOffset in range (0,CA_SIZE):
            toNeuron = toOffset + (toCA*CA_SIZE)
            connector = connector + [(0,toNeuron,INPUT_WEIGHT,
                                      DELAY)]
        nealProjection(spikeSource, toNeurons, connector,'excitatory')

def makeCA(neurons, CA):
        #print 'makeCA
        connector = []
        for fromOffset in range (0,CA_SIZE):
            fromNeuron = fromOffset + (CA*CA_SIZE)
            for toOffset in range (0,CA_SIZE):
                toNeuron = toOffset + (CA*CA_SIZE)
                if (toNeuron != fromNeuron):
                    connector = connector + [(fromNeuron,toNeuron,
                                              INTRA_CA_WEIGHT,
                                              DELAY)]

        nealProjection(neurons,neurons,connector,'excitatory')

#--Functions when two inputs are needed to turn on a third
#-- Two states are needed to turn on a third.  
#-- This connects one of the inputs to the the third.
#-- Uses HALF_ON_WEIGHT
def stateHalfTurnsOnState(fromNeurons,fromCA,toNeurons,toCA):
    connector = []
    #uses HALF_ON_WEIGHT
    for fromOffset in range (0,CA_SIZE):
        fromNeuron = fromOffset + (fromCA*CA_SIZE)
        for toOffset in range (0,CA_SIZE):
            toNeuron = toOffset + (toCA*CA_SIZE)
            connector=connector+[(fromNeuron,toNeuron,HALF_ON_WEIGHT, DELAY)]

    nealProjection(fromNeurons,toNeurons,connector,'excitatory')

#---- One State or other set of neurons turns off another
#-- Uses CA_STOPS_CA_WEIGHT
def stateTurnsOffState(fromNeurons, fromCA, toNeurons, toCA):
    connector = []
    for fromOffset in range (0,CA_SIZE):
        fromNeuron = fromOffset + (fromCA*CA_SIZE)
        for toOffset in range (0,CA_SIZE):
            toNeuron = toOffset + (toCA*CA_SIZE)
            connector = connector + [(fromNeuron,toNeuron,CA_STOPS_CA_WEIGHT,
                                              DELAY)]

    nealProjection(fromNeurons,toNeurons,connector,'inhibitory')

def connectPoissonToNeurons(spikeSource,sourceNum, neurons, toCA):
    connector = []
    for toOffset in range (0,2):
        toNeuron = toOffset + (toCA * CA_SIZE)
        connector = connector + [(sourceNum,toNeuron, 2.2, DELAY)]

    nealProjection(spikeSource,neurons,connector,'excitatory')

#-----Main


#initSim()

#source = sim.Population(1,sim.PoissonGenerator)
poissonParams = {'rate':5.0}

source = sim.Population(4,sim.SpikeSourcePoisson,poissonParams)
circuit = createMyNeurons()

#connectPoissonToNeurons(source,0,circuit,0)
#connectPoissonToNeurons(source,1,circuit,1)
#connectPoissonToNeurons(source,2,circuit,2)
#connectPoissonToNeurons(source,3,circuit,3)

#makeCA(circuit,0)
#makeCA(circuit,1)
#makeCA(circuit,2)
#makeCA(circuit,3)
#stateHalfTurnsOnState(circuit,0,circuit,1)
#stateHalfTurnsOnState(circuit,1,circuit,2)
#stateHalfTurnsOnState(circuit,2,circuit,3)
#stateHalfTurnsOnState(circuit,3,circuit,1)
#stateTurnsOffState(circuit,1,circuit,0)
#stateTurnsOffState(circuit,2,circuit,0)
#stateTurnsOffState(circuit,3,circuit,0)
#stateTurnsOffState(circuit,2,circuit,1)
#stateTurnsOffState(circuit,3,circuit,2)
#stateTurnsOffState(circuit,1,circuit,3)
